function [UD] = hcluster_all(X, type, method)
% Hierarchical clustering.
% Returns the ultrametric distance matrix resulting from the clustering.
%
% X: data points (rows of X) OR D distance matrix of the points
% type: 0 if X = data points as rows
%       1 if X = distance matrix
% method: hierarchical clustering method: 'single', 'complete', 'average'
%
% (c) 2011 Zalan Bodo

% Performing the hierarchical clustering using 'linkage'
if type == 0
    N = size(X,1);
    Z = linkage(X, method, 'euclidean');
    %Z = linkage(full(X), method, 'euclidean');
else
    if type == 1
        N = size(X,1);
        Z = linkage(squareform(X), method);
    end
end

% Initialization
UD = zeros(N, N);

I = struct; % This structure will be used as an auxiliary variable to store the clustered points

% Creating a 'ci' field for each point x_i
for i=1:N
    I = setfield(I, ['c' num2str(i)], i);
end

% Setting distances, creating UD
for i=1:N-1
    i1  = ['c' num2str(Z(i,1))]; i2 = ['c' num2str(Z(i,2))];
    J = getfield(I, i1); K = getfield(I, i2);
    I = setfield(I, ['c' num2str(i+N)], [J K]); % Merging the clusters
    I = rmfield(I, {i1, i2}); % Removing previous sets
    [X,Y] = meshgrid(J, K);
    X = [reshape(X, size(X,2)*size(X,1), 1)];
    Y = [reshape(Y, size(Y,2)*size(Y,1), 1)];
    UD(sub2ind([N N], X, Y)) = Z(i,3); % Setting distance
    UD(sub2ind([N N], Y, X)) = Z(i,3); % Setting distance
end
